package com.pms.ocp.biz.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.pms.ocp.biz.BizModelRequirementService;
import com.pms.ocp.model.dto.ModelPropertyVersionDemandDTO;
import com.pms.ocp.model.entity.*;
import com.pms.ocp.model.dto.ModelVersionDemandDTO;
import com.pms.ocp.service.ModelBaseVersionService;
import com.pms.ocp.service.ModelDemandService;
import com.pms.ocp.service.ModelPropertyVersionService;
import com.pms.ocp.service.OcpModelGroupService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * @Auther: wangjian
 * @Date: 2022/4/1 11:19
 * @Description:
 */

@Service
public class BizModelRequirementServiceImpl implements BizModelRequirementService {

    @Autowired
    private ModelDemandService modelDemandService;

    @Autowired
    private ModelBaseVersionService modelBaseVersionService;

    @Autowired
    private ModelPropertyVersionService modelPropertyVersionService;


    @Transactional
    @Override
    public Integer createModelRequirement(ModelVersionDemandDTO modelVersionDemandDTO) {

        ModelRequirement modelRequirement = new ModelRequirement();
        BeanUtils.copyProperties(modelVersionDemandDTO, modelRequirement);
        modelRequirement.setCtime(LocalDateTime.now());

        ModelBaseVersion modelBaseVersion = new ModelBaseVersion();
        BeanUtils.copyProperties(modelVersionDemandDTO, modelBaseVersion);
        modelBaseVersionService.createModelBaseVersion(modelBaseVersion);

        modelDemandService.createModelDemand(modelRequirement);

        return 1;
    }

    @Override
    public Integer createModelPropertyRequirement(ModelPropertyVersionDemandDTO modelPropertyVersionDemandDTO) {

        ModelRequirement modelRequirement = new ModelRequirement();
        BeanUtils.copyProperties(modelPropertyVersionDemandDTO, modelRequirement);
        modelRequirement.setCtime(LocalDateTime.now());

        ModelPropertyVersion modelPropertyVersion = new ModelPropertyVersion();
        BeanUtils.copyProperties(modelPropertyVersionDemandDTO, modelPropertyVersion);
        modelPropertyVersionService.createModelPropertyVersion(modelPropertyVersion);

        modelDemandService.createModelDemand(modelRequirement);

        return 1;
    }

    @Transactional
    @Override
    public Integer updateModelRequirement(ModelVersionDemandDTO modelVersionDemandDTO) {
        ModelRequirement modelRequirement = new ModelRequirement();
        BeanUtils.copyProperties(modelVersionDemandDTO, modelRequirement);
        modelRequirement.setMtime(LocalDateTime.now());

        ModelBaseVersion modelBaseVersion = new ModelBaseVersion();
        BeanUtils.copyProperties(modelVersionDemandDTO, modelBaseVersion);
        modelBaseVersionService.updateModelBaseVersion(modelBaseVersion);

        modelDemandService.updateModelDemand(modelRequirement);
        return 1;
    }

    @Override
    public Integer updateModelPropertyRequirement(ModelPropertyVersionDemandDTO modelPropertyVersionDemandDTO) {
        ModelRequirement modelRequirement = new ModelRequirement();
        BeanUtils.copyProperties(modelPropertyVersionDemandDTO, modelRequirement);
        modelRequirement.setMtime(LocalDateTime.now());

        ModelPropertyVersion modelPropertyVersion = new ModelPropertyVersion();
        BeanUtils.copyProperties(modelPropertyVersionDemandDTO, modelPropertyVersion);
        modelPropertyVersionService.updateModelPropertyVersion(modelPropertyVersion);


        modelDemandService.updateModelDemand(modelRequirement);
        return 1;
    }

    @Transactional
    @Override
    public Integer deleteModelRequirement(String demandId, String modelId) {

        modelDemandService.deleteModelDemand(modelId, demandId);

        if (StringUtils.isNotEmpty(modelId)) {

            modelBaseVersionService.deleteModelBaseVersion(modelId, demandId);
        }

        return 1;
    }

    @Override
    public Integer deleteModelPropertyRequirement(String demandId, String modelId, String proId) {
        modelDemandService.deleteModelDemand(modelId, demandId);

        if (StringUtils.isNotEmpty(proId)) {

            modelPropertyVersionService.deleteModelPropertyVersion(demandId, proId);
        }

        return 1;
    }


    @Autowired
    private OcpModelGroupService ocpModelGroupService;

    @Override
    public Model getModelRequirement(String objId, String modelGroupCode) {
        return null;
    }

    @Override
    public Model getModelPropertyRequirement(String objId, String modelGroupCode) {
        return null;
    }

    @Override
    public List<ModelBaseVersion> getModelRequirementList(String modelCode, String modelGroupCode, int currentPage, int pageSize) {

        List<String> modelGroupCodeList = new ArrayList<>();

        QueryWrapper<ModelClassify> queryWrapper = new QueryWrapper();
        List<ModelClassify> modelClassifyList = ocpModelGroupService.list(queryWrapper);

        getParentNode(modelGroupCodeList, modelClassifyList, modelGroupCode);
        modelGroupCodeList.add(modelGroupCode);

        List<ModelBaseVersion> modelBaseVersionList = modelBaseVersionService.getModelBaseVersionList(modelCode, modelGroupCodeList, currentPage, pageSize);

        return modelBaseVersionList;
    }

    @Override
    public List<ModelPropertyVersion> getModelPropertyRequirementList(String propId, String modelCode, int currentPage, int pageSize) {

        List<ModelPropertyVersion> modelPropertyVersionList = modelPropertyVersionService.getModelPropertyVersionList(propId, modelCode);

        return modelPropertyVersionList;
    }

    /**
     * 递归获取某个父节点下面的所有子节点
     *
     * @param childMenu       要返回的结果
     * @param menuList        数据库查询出来的所有集合
     * @param modelGroupPcode 父类编码
     */
    private static void getParentNode(List<String> childMenu, List<ModelClassify> menuList, String modelGroupPcode) {
        menuList.stream()
                //过滤出父id等于参数的id
                .filter(menu -> StringUtils.isNotBlank(menu.getModelGroupPcode()) && menu.getModelGroupPcode().equals(modelGroupPcode))
                .forEach(menu -> {
                    //递归遍历下一级
                    getParentNode(childMenu, menuList, menu.getModelGroupCode());
                    //添加
                    childMenu.add(menu.getModelGroupCode());
                });
    }

}
